#!/usr/bin/php -q
<?php
#
# /////////////////////////////////////////////////////////////////////////////////////////////
# //                              listserv2mbox version 1.2                                  //
# /////////////////////////////////////////////////////////////////////////////////////////////
# 
# Tool name:	listserv2mbox
# Description:	PHP command-line (CLI) script that converts LISTSERV digests into standard Linux mbox (mail spool) format
# Author:	Joey Kelly, email joey@joeykelly.net, web http://joeykelly.net
# Date:		November 17, 2003
# License:	GPL
# Usage:	listserv2mbox-nospam.ph < listserv_digest.txt > mbox        # unix bash shell syntax --- windows users are on their own
# Tips:		Run hypermail on the resulting mbox file to create web archives
# Changelog:	Version 1.2: Support for munging of addresses embedded in the body of each email
# 		Version 1.1: Mail headers are munged to foil spambot email harvesting agents
#
#
#######################################################################################
#######################################################################################
# ADDRESS MUNGING: WE ARE TRYING TO MUNGE THE ADDRESSES TO STOP SPAMBOTS FROM LEECHING ADDRESSES
# Version 1.1 only dealt with the headers. Embedded email addresses were ignored.
# Version 1.2 tried to handle email addresses found in the body of each email.
#
# Perfection is not the goal here. A success rate of 90% is my target for these 2 releases.
# If someone wants to improve my algorithms, they are welcome to do so.
# I know the methods I used aren't that great, but again, perfection wasn't demanded by my client.
#
# --Joey Kelly
#
#######################################################################################
#######################################################################################
# Algorithm:
# 1. read in a line from the LISTSERV file
# 2. while line !== "===========" # (whatever length it is) grab another line   ### this removes the user's mail client headers, LISTSERV comments, etc
# 3. start parsing the headers for the needed info
# 4. print the corrected headers to the target mbox file
# 5. while line !== "===========" print to the target mbox file
#
#
#
#######################################################################################
#######################################################################################
# here's what we need, per mbox specs: 
# $from
# $returnpath  # deduced from $from
# $reveived    # construct this
# $date
# $shortfrom
# $msgid
# $to
# $subject
# 
# We can turn off the messageID field in hypermail, so let's put some bogus data there
#
# After that, we can echo the rest of the message
#
#
# NOTE: All of the archives the author has checked seems to be in a non-varying format. YMMV.
#
#
# We need to watch for embedded code: addslashes() will do this for us
#
#######################################################################################
#######################################################################################
#
#
# First we disable max_execution time, enabling us to take as much time as needed to parse huge files
# Uncomment this line as needed
#set_time_limit(0);
#
# Here we define $stdin and $stdout.
# We can use the regular CLI constants STDIN and STDOUT, but we can't depend on $user having these defined for him
$stdin = fopen('php://stdin', 'r');
$stdout = fopen('php://stdout', 'w');
#
$newmail 	=  "========================================================================="; 
#
#
# grab the first line
# We need to catch any headers, LISTSERV routing data, etc. from client's mail reader
$line = addslashes(trim(fgets($stdin, 4096)));
while ((!feof ($stdin)) and (!($line == $newmail))) {
  $line = addslashes(trim(fgets($stdin, 4096)));
}
while (!feof ($stdin)) {
  if ($line == $newmail) {
    # Here we read in each line of digest header data
    $Adate	= addslashes(trim(fgets($stdin, 4096)));
    $Areply	= addslashes(trim(fgets($stdin, 4096)));
    $Asender	= addslashes(trim(fgets($stdin, 4096)));	# We don't need this, but we do have to read in this line
    $Afrom	= addslashes(trim(fgets($stdin, 4096)));
    $Asubject	= addslashes(trim(fgets($stdin, 4096)));
    $Amime	= addslashes(trim(fgets($stdin, 4096)));	# Do we need this?
    $Acontent	= addslashes(trim(fgets($stdin, 4096)));
    #
    # Here we are going to manipulate the headers to conform to mbox format
    # First we list and/or define the variables we need to end up with
    $from	= $Afrom;
    $return	= $Afrom;
    $received	= $received;	# constructed from $from
    $date	= $Adate;
    $shortfrom	= $Afrom;
    $msgid	= "Message-Id: <200305122255.h4CMt1U02501@example.com>";	# manufacture this, maybe this string works??
    $to		= $to;		# manufacture this
    $subject	= $Asubject;
    #
    #
    # Now we are going to manipulate the data to end up with each variable in the correct format
    #
    # Working on $from	---> do $shortfrom and $date first, then construct $from
    #
    # Working on $return	---> do $shortfrom, then construct $return
    #
    # Working on $date
    $badchar		= chr(160);
    $date	= str_replace($badchar, "", $Adate);
    $datepattern        = "/Date:\s*/";
    $datereplacement    = "";
    $justdate	= preg_replace($datepattern, $datereplacement, $date);
    $date	= "Date: $justdate";
    #
    # Working on $received
    $received	= "Received: (from root@localhost)
        by example.com (8.11.6/8.11.6/SuSE Linux 0.5) id h4CMt1U02501
        for root; Mon, 12 May 2003 17:55:01 -0500";		# blah, we may just need a placeholder here
    # Working on $shortfrom
    $badchar	= chr(160);
    $shortfrom	= str_replace($badchar, "", $Afrom);
    $pattern	= "/From:\s*/";
    $replacement	= "From: ";
    $shortfrom	= preg_replace($pattern, $replacement, $shortfrom);
    #
    #################################################################################
    # Here we have to munge any email addresses we find in the headers of the email #
    #################################################################################
    #
    $at         = strpos($shortfrom, "@");
    $shortfrom  = substr($shortfrom, 0, $at);
    $shortfrom  .= "@xxxxxxxx.xxx>";
    #
    # Working on $to
    $to		= "To: user@example.com";
    #
    # Working on $subject
    $badchar	= chr(160);
    $subject	= str_replace($badchar, "", $Asubject);
    $pattern	= "/Subject:\s*/";
    $replacement	= "Subject: ";
    $subject	= preg_replace($pattern, $replacement, $subject);
    $subject	= stripslashes($subject);	# Here we remove the slashes we added earlier when ckecking user input
    #
    #
    # Now we go back and work on $from
    $openbracket	= strpos($shortfrom, "<") + 1;
    $closebracket	= strpos($shortfrom, ">") - 1;
    $lengthemail	= $closebracket - $openbracket + 1;	#adding back the 1 we subtracted to lose the ">" character
    $justfrom	= substr($shortfrom, $openbracket, $lengthemail);
    $fromdate	= str_replace(",", "", $justdate);	# assuming I've gotten this far...
    $dateparts	= explode(" ", $fromdate);
    $fromdate	= "$dateparts[0] $dateparts[2] $dateparts[1] $dateparts[4] $dateparts[3]";	# here we rearrange the date string
    $from	= "From $justfrom  $fromdate";
    #
    # Now we go back and work on $return
    $return	= "Return-Path: <$justfrom>";		# we are putting the brackets back around the address. Sigh.
    # 
    # This echoes our completed headers
    $headers	= "$from\n$return\n$received\n$date\n$shortfrom\n$msgid\n$to\n$subject\n";
    fwrite($stdout, $headers);
    #
    # Here we grab each line of the body of the email.
    $line	= "";	# This resets $line so we can enter the following loop
    while ((!feof ($stdin)) and (!($line == $newmail))) {
      $line = addslashes(trim(fgets($stdin, 4096)));
      if (!($line == $newmail)) {
        $line	= stripslashes($line);
        #
        ##############################################################################
        # Here we have to munge any email addresses we find in the body of the email #
        ##############################################################################
        #
        # If we find an "@" in the line, we suspect it contains an address, so we process it
        $atpos = strpos($line, '@');
        if (!($at === false)) {
          # Here we detect valid email addresses and substitute our munged version into $line
          #
          # This really needs to be an in-depth verifying process... but email is hard to regex.
          # If someone wants to improve on this, they are welcome to do it.
          # For now, though, I'm going to do a basic set of tests, and try for 90% accuracy
          #
          # Here's what I need to regex for:
          # "@", one or more of "lettersnumberswhatothercharacters? and a dot", and "letters"
          #
          $line = preg_replace("/(@[-0-9a-zA-Z]+\.)+[a-zA-Z]+/", "@xxx.yyy", $line);
        }
        #
        # Ok, we're done, so let's print the line
        fwrite($stdout, "$line\n");
      }
    }
  }    
fwrite($stdout, "\n\n");
}
#
# We are done :-)
#
?>
